/*
   Keyword - einfaches Suchprogramm fuer Stichworte

      ... kein perfektes Programm, mehr eine Anregung fuer
      all diejenigen, die Lust und Laune haben mehr daraus
      zu machen!

   Hinweis: MySQL-Datenbank und Anwender muessen bestehen 
            (stichworte.sql)

   Autor..: Karsten Brodmann <kb@punkt-akademie.de>
   Version: 0.1
   Datum..: 2017-11-01
*/
import java.sql.*;
import org.apache.commons.cli.*;

public class Keyword {
   // JDBC-Treiber Verbindungsdaten
   static final String JDBCDRIVER = "com.mysql.jdbc.Driver";  
   static final String DBURL = "jdbc:mysql://localhost/stichworte?useSSL=false";
   static final String USER = "stwsucher";
   static final String PASS = "1234";

   // Datenbankverbindung
   Connection conn = null;
   Statement stmt = null;
   ResultSet rs = null;
   // Programmparameter
   String searchfor;
   boolean booleanmode = false;


   // Benutzerhinweise
   private static void printUsage(Options opts)
   {
      HelpFormatter formatter = new HelpFormatter();
      formatter.printHelp("Keyword", opts);
      System.exit(1);
   }


   // DB-Verbindung herstellen
   private void getConnection() throws ClassNotFoundException, SQLException
   {
      Class.forName(JDBCDRIVER);
      conn = DriverManager.getConnection(DBURL,USER,PASS);
   }

   
   // Programmoptionen auslesen, im Fehlerfall Programm beenden
   private void getOptions(String[] args)
   {
      Options options = new Options();

      Option searchstring = new Option("s", "search", true, "searchstring");
      searchstring.setRequired(true);
      options.addOption(searchstring);

      Option booleansearch = new Option("b", "bool", false, "boolean search");
      booleansearch.setRequired(false);
      options.addOption(booleansearch);

      CommandLineParser parser = new DefaultParser();
      CommandLine cmd = null;

      try {
         cmd = parser.parse(options, args);
      } catch (ParseException e) {
         System.out.println(e.getMessage());
         printUsage(options);  // beendet Programm
      }

      searchfor = cmd.getOptionValue("s");
      if(cmd.hasOption("b")) {
         booleanmode = true;
      }
   }


   // SSQL-Statement basteln
   private String createSQL(String searchfor, boolean booleanmode) 
   {
      String sql;

      sql =       "SELECT stichwort, beschreibung, lektion ";
      sql = sql + "FROM stichworte ";
      sql = sql + "WHERE MATCH (stichwort, beschreibung) ";
      sql = sql + "AGAINST ('" + searchfor;
      if (booleanmode) {
         sql = sql + "' IN BOOLEAN MODE)";
      } else {
         sql = sql + "' IN NATURAL LANGUAGE MODE)";
      }

      return sql;
   }


   // Ergebnisse ausgeben
   private void printResult(int no, String kw, String desc, String lec)
   {
      String hline = "----------------------------------------";

      if (no == 1) {
         System.out.println(hline);
      }
      System.out.println("Treffer.....: " + no);
      System.out.println("Stichwort...: " + kw);
      System.out.println("Beschreibung: " + desc);
      System.out.println("Lektion.....: " + lec);
      System.out.println(hline);
   }


   // Ergebnisse aus DB abrufen
   private void getResults() throws SQLException
   {
      stmt = conn.createStatement();
      rs = stmt.executeQuery(createSQL(searchfor, booleanmode));
      int no = 0;
      while(rs.next()) {
         no++;
         printResult(no, rs.getString("stichwort"), 
                         rs.getString("beschreibung"), 
                         rs.getString("lektion"));
      }
      System.out.println("(" + no + " Treffer insgesamt)" );
      rs.close();
   }


   // Aufraeumarbeiten
   private void cleanup() {
      try {
         if (stmt!=null)
            stmt.close();
      } catch(SQLException e) {
         e.printStackTrace();
      }
      try {
         if (conn!=null)
            conn.close();
      } catch(SQLException e) {
         e.printStackTrace();
      }
   }


   // Hauptprogramm
   public static void main(String[] args) 
   {
      Keyword kw = new Keyword();

      kw.getOptions(args);
   
      try {
         kw.getConnection();
         kw.getResults();
         kw.cleanup();
      } catch(SQLException e) {
         // JDBC-Fehler abfangen
         e.printStackTrace();
      } catch(Exception e) {
         // andere Fehler abfangen
         e.printStackTrace();
      } finally {
         // auf jeden Fall aufraeumen
         kw.cleanup();
      }
   }
}